package cn.yufire.config.core;


import lombok.Data;

/**
 * @author Yufire
 * @date 2021/3/17 16:32
 * @description 全局唯一核心配置类
 */
@Data
public class AppConfiguration {

    /**
     * 用户名
     */
    private String userName;
    /**
     * 用户年龄
     */
    private Integer userAge;
    /**
     * 是否喜欢吃鸡蛋
     */
    private Boolean userIsEatEgg;
    /**
     * 分数
     */
    private Double score;


    /**
     * 全局唯一对象
     * 第二层锁，volatile关键字禁止指令重排
     */
    private volatile static AppConfiguration config = null;

    /**
     * 懒汉式
     */
    private AppConfiguration() {
    }

    public static AppConfiguration getInstance() {
        // 第一层检查，检查是否有引用指向对象，高并发情况下会有多个线程同时进入
        if (config == null) {
            // 第一层锁，保证只有一个线程进入
            // 双重检查，防止多个线程同时进入第一层检查(因单例模式只允许存在一个对象，故在创建对象之前无引用指向对象，所有线程均可进入第一层检查)
            // 当某一线程获得锁创建一个AppConfiguration对象时,即已有引用指向对象，config不为空，从而保证只会创建一个对象
            // 假设没有第二层检查，那么第一个线程创建完对象释放锁后，后面进入对象也会创建对象，会产生多个对象
            synchronized (AppConfiguration.class) {
                // 第二层检查
                if (config == null) {
                    // config = new AppConfiguration() 语句为非原子性，实际上会执行以下内容：
                    // (1)在堆上开辟空间；(2)属性初始化;(3)引用指向对象
                    // volatile关键字可保证config = new AppConfiguration()语句执行顺序为123，
                    config = new AppConfiguration();
                }
            }
        }
        return config;
    }


}
