package cn.yufire.config.core;

import lombok.extern.slf4j.Slf4j;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author Yufire
 * @date 2021/3/3 09:31
 * @description 刷新类
 */
@Slf4j
public class Refresh {

    /**
     * 刷新配置
     *
     * @param configurationStr 配置字符串
     * @param configName       配置名称
     */
    public static void refresh(String configurationStr, String configName) {
        System.out.printf("******** 正在同步：%s ********\n", configName);
        fillProperties(parsingProperties(configurationStr));
    }

    /**
     * 解析配置文件方法
     *
     * @param configurationStr 配置字符串 xxx.properties类型 格式要求严格
     */
    private static Map<String, Object> parsingProperties(String configurationStr) {
        try {
            Properties properties = getProperties(configurationStr);
            if (ObjectUtil.isNull(properties) || properties.size() <= 0) {
                log.error("配置文件读取失败！");
                return null;
            }
            // 存储解析后配置的Map  (key,val)
            Map<String, Object> propertiesMap = new HashMap<>(properties.size());
            // 2. 填充map
            for (String propertiesName : properties.stringPropertyNames()) {
                // 填充时的key名转小写(更方便匹配)
                propertiesMap.put(propertiesName.toLowerCase(), properties.getProperty(propertiesName));
            }
            return propertiesMap;
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("********同步失败********");
        }

        return null;
    }

    /**
     * 填充数据至配置类
     *
     * @param propertiesMap 配置集合
     */
    private static void fillProperties(Map<String, Object> propertiesMap) {
        if (CollUtil.isEmpty(propertiesMap)) {
            log.error("读取到的配置Map为空不加载配置！");
            return;
        }
        // 获取单例config对象
        AppConfiguration instance = AppConfiguration.getInstance();
        // 因为要调用该对象的setXXX方法 所以要获取该对象的所有方法列表
        Method[] methods = instance.getClass().getDeclaredMethods();
        // 循环每一个方法
        for (Method method : methods) {
            // 参数个数
            int parameterCount = method.getParameterCount();
            // set方法的参数只会有一个 添加方法形参数量为1的判断
            if (parameterCount == 1) {
                // 获取参数类型 用于执行setXXX方法是时的形参类型转换     // 只有一个参数 所有获取第0个下标
                Class<?> parameterType = method.getParameterTypes()[0];
                // 获取该字段的名称 Lombok生成的setXXX方法都是以驼峰的形式命名的 所以只需把set去掉剩下的就是该字段名 转小写和Map里的参数匹配上
                String filedName = method.getName().replace("set", "").toLowerCase();
                // 要给该字段赋的值
                Object val = propertiesMap.get(filedName);
                if (ObjectUtil.isNotNull(val)) {
                    // 类型转换
                    if (parameterType.equals(String.class)) {
                        val = Convert.toStr(val);
                    } else if (parameterType.equals(Integer.class)) {
                        val = Convert.toInt(val);
                    } else if (parameterType.equals(Double.class)) {
                        val = Convert.toDouble(val);
                    } else if (parameterType.equals(Boolean.class)) {
                        val = Convert.toBool(val);
                    } else {
                        val = Convert.toStr(val);
                    }
                    // 执行setXXX方法
                    try {
                        method.invoke(instance, val);
                    } catch (Exception e) {
                        log.error("字段类型转换错误，或setXXX方法未匹配成功！");
                    }
                }
            }
        }
        System.out.println("******** 同步完成 ********");
    }


    /**
     * 获取Properties对象
     *
     * @param configurationStr 配置字符串
     * @return
     */
    private static Properties getProperties(String configurationStr) {
        if (StrUtil.isEmpty(configurationStr)) {
            log.error("配置文件为空不加载配置");
            return null;
        }
        // 1. 解析配置
        InputStream inputStream = new ByteArrayInputStream(configurationStr.getBytes());
        Properties properties = new Properties();
        try {
            // 使用 InputStreamReader 防止中文乱码
            properties.load(new InputStreamReader(inputStream));
        } catch (Exception e) {
            log.error("解析配置失败！");
            return null;
        }
        return properties;
    }

}
